#pragma once

#include <cstdint>

#include "common/fs.h"
#include "crypto/hash.h"
#include "cryptonote_basic/cryptonote_basic.h"
#include "cryptonote_core/cryptonote_core.h"

namespace cryptonote {

class Snapshot {
public:
    explicit Snapshot(cryptonote::network_type nettype) : m_nettype{nettype} {}

    bool replace_databases(const fs::path &config_dir, const fs::path &sqlite_db_path, const fs::path &ons_db_path);

    uint64_t get_height() const { 
        return get_config(m_nettype).SNAPSHOT_HEIGHT;
    }

private:
    cryptonote::network_type m_nettype;
    std::string snapshot_file = "snapshot.zip";
    fs::path prepared_snapshot_dir;

    // Helper function that downloads and prepares the snapshot once.
    bool prepare_snapshot();

    // Helper methods for downloading and processing the snapshot
    bool download_verify_and_unzip(fs::path &tmp_unzip);
    bool download_snapshot(const std::string& url, const std::string& out_file);
    bool verify_checksum(const std::string& file_path, const std::string& expected_checksum);
    bool unzip_snapshot(const std::string& zip_file, const std::filesystem::path& dest_dir);
    bool replace_dbs(const fs::path& source_dir, const fs::path& config_dir, const fs::path& target_sqlite, const fs::path& target_ons);
};

}
